jQuery(document).ready(function($) {
    // -------------------- Cached jQuery Selectors --------------------
    const $spinner = $('#aipower-spinner');
    const $message = $('#aipower-message');
    const $pdfDetailsContainer = $('#aipower-pdf-details-container');
    const $pdfsTableContainer = $('#aipower-pdfs-table-container');
    const $nonceField = $('#ai-pdfs-nonce');
    const nonce = $nonceField.val();
    const $searchInput = $('#aipower-pdfs-search-input');
    const $deleteProgress = $('#aipower-delete-progress');
    const $deleteProgressCounter = $('#aipower-delete-progress-counter');
    let searchTimer;

    // -------------------- UI FEEDBACK: Spinner and Message Display Functions --------------------
    const UI = {
        showSpinner: () => $spinner.show(),
        hideSpinner: () => $spinner.hide(),
        showMessage: (type, text, autoHide = false) => {
            $message
                .removeClass('error success aipower-autosaving')
                .addClass(type)
                .text(text)
                .fadeIn();

            if (autoHide) {
                setTimeout(() => $message.fadeOut(), 3000);
            }
        }
    };

    // -------------------- Event Handlers --------------------

    // Toggle the tools menu when the tools icon is clicked
    $('#aipower-pdf-tools-icon').on('click', function(event) {
        event.stopPropagation(); // Prevent the click from propagating to the document
        $('#aipower-pdf-tools-menu').toggle();
    });

    // Hide the tools menu when clicking outside
    $(document).on('click', function(e) {
        if (!$(e.target).closest('#aipower-pdf-tools-container').length) {
            $('#aipower-pdf-tools-menu').hide();
        }
    });

    // Search Input Handler
    $searchInput.on('input', function() {
        clearTimeout(searchTimer);
        const searchTerm = $(this).val();

        // Delay the AJAX call to prevent too many requests
        searchTimer = setTimeout(() => {
            UI.showSpinner();
            // Send AJAX request to refresh PDFs table with search term
            $.ajax({
                url: ajaxurl,
                type: 'POST',
                data: {
                    action: 'aipower_refresh_pdfs_table',
                    page: 1, // Reset to first page on new search
                    search_term: searchTerm,
                    _wpnonce: nonce
                },
                success: function(response) {
                    UI.hideSpinner();
                    if (response.success) {
                        $pdfsTableContainer.html(response.data.table);
                        UI.showMessage('success', response.data.message || 'PDFs table updated.', true);
                    } else {
                        UI.showMessage('error', response.data.message, true);
                    }
                },
                error: function() {
                    UI.hideSpinner();
                    UI.showMessage('error', 'An error occurred while refreshing the PDFs table.', true);
                }
            });
        }, 300); // Adjust delay as needed
    });

    // Refresh PDFs Table from the tools menu
    $(document).on('click', '#aipower-pdf-refresh', function() {
        $('#aipower-pdfs-search-input').val(''); // Clear search input
        $('#aipower-pdf-tools-menu').hide(); // Hide the menu after action
        UI.showSpinner();
        // Send AJAX request to refresh PDFs table
        $.ajax({
            url: ajaxurl,
            type: 'POST',
            data: {
                action: 'aipower_refresh_pdfs_table',
                page: 1, // Refresh to the first page
                _wpnonce: nonce
            },
            success: function(response) {
                UI.hideSpinner();
                if (response.success) {
                    $pdfsTableContainer.html(response.data.table);
                    UI.showMessage('success', response.data.message || 'PDFs table updated.', true);
                } else {
                    UI.showMessage('error', response.data.message, true);
                }
            },
            error: function() {
                UI.hideSpinner();
                UI.showMessage('error', 'An error occurred while refreshing the PDFs table.', true);
            }
        });
    });

    // Show confirmation prompt when Delete All is clicked
    $(document).on('click', '#aipower-pdf-delete-all', function(event) {
        event.stopPropagation(); // Prevent the click from propagating to the document
        // Toggle the visibility of the confirmation prompt
        $('#aipower-pdf-confirmation').toggle();
        // Hide the tools menu
        $('#aipower-pdf-tools-menu').hide();

        if ($('#aipower-pdf-confirmation').is(':visible')) {
            // Attach a click handler to the document to hide the confirmation prompt when clicking outside
            $(document).on('click.aipower', function(e) {
                if (!$(e.target).closest('#aipower-pdf-confirmation').length && !$(e.target).is('#aipower-pdf-tools-icon')) {
                    $('#aipower-pdf-confirmation').hide();
                    // Remove this click handler to prevent multiple bindings
                    $(document).off('click.aipower');
                }
            });
        } else {
            // If the confirmation prompt is hidden, remove the document click handler
            $(document).off('click.aipower');
        }
    });

    // Prevent clicks inside the confirmation prompt from propagating to the document
    $(document).on('click', '#aipower-pdf-confirmation', function(event) {
        event.stopPropagation();
    });

    // Handle confirmation "Yes" click for Delete All
    $(document).on('click', '#aipower-pdf-confirm-yes', function() {
        $('#aipower-pdf-confirmation').hide();
        $(document).off('click.aipower');
    
        // 1. First, ask server for ALL PDF IDs.
        UI.showSpinner();
        $.ajax({
            url: ajaxurl,
            type: 'POST',
            data: {
                action: 'aipower_get_all_pdf_ids',
                _wpnonce: nonce
            },
            success: function(response) {
                UI.hideSpinner();
                if (response.success) {
                    const pdfIds = response.data.ids || [];
                    if (pdfIds.length === 0) {
                        // No PDFs at all
                        UI.showMessage('error', 'No PDFs found to delete.', true);
                    } else {
                        // 2. We have an array of all PDF IDs. Let's start deleting them one by one.
                        startDeletingAllPDFs(pdfIds);
                    }
                } else {
                    UI.showMessage('error', response.data.message, true);
                }
            },
            error: function() {
                UI.hideSpinner();
                UI.showMessage('error', 'An error occurred fetching PDF IDs.', true);
            }
        });
    });

    // This function deletes PDFs sequentially, one by one:
    function startDeletingAllPDFs(pdfIds) {
        // Initialize progress display
        const total = pdfIds.length;
        let deleted = 0;

        // Show the progress bar (assuming you have a hidden element #aipower-delete-progress)
        $deleteProgress.show();
        $deleteProgressCounter.text(`Deleting 0/${total}...`);

        // We'll define a recursive function that deletes the next PDF in the array
        const deleteNext = () => {
            // If pdfIds array is empty, we've deleted all
            if (pdfIds.length === 0) {
                UI.showMessage('success', `All ${total} PDFs deleted successfully.`, true);
                $deleteProgress.hide();
                // Optionally clear the table and details container
                $pdfsTableContainer.html('');
                $pdfDetailsContainer.empty();
                return;
            }

            // Pop the first PDF ID
            const currentId = pdfIds.shift();
            deleted++;

            // Update progress text
            $deleteProgressCounter.text(`Deleting ${deleted}/${total}...`);

            // Show spinner during each delete request
            UI.showSpinner();
            $.ajax({
                url: ajaxurl,
                type: 'POST',
                data: {
                    action: 'aipower_delete_pdf',
                    pdf_id: currentId,
                    _wpnonce: nonce
                },
                success: function(resp) {
                    UI.hideSpinner();
                    if (!resp.success) {
                        // We can show an error, but continue deleting the rest
                        UI.showMessage('error', resp.data.message || 'Error deleting a PDF.', true);
                    }
                },
                error: function() {
                    UI.hideSpinner();
                    UI.showMessage('error', `Error deleting PDF ID: ${currentId}`, true);
                },
                complete: function() {
                    // Move on to the next ID
                    deleteNext();
                }
            });
        };

        // Start the first deletion
        deleteNext();
    }

    // View PDF Details
    $(document).on('click', '.aipower-view-pdf-icon', function() {
        const pdf_id = $(this).data('id');

        UI.showSpinner();
        // Send AJAX request to load PDF details
        $.ajax({
            url: ajaxurl,
            type: 'POST',
            data: {
                action: 'aipower_load_pdf_details',
                pdf_id: pdf_id,
                _wpnonce: nonce
            },
            success: function(response) {
                UI.hideSpinner();
                if (response.success) {
                    $pdfDetailsContainer.html(response.data.pdf_details);
                    UI.showMessage('success', response.data.message || 'PDF details loaded.', true);
                } else {
                    UI.showMessage('error', response.data.message, true);
                }
            },
            error: function() {
                UI.hideSpinner();
                UI.showMessage('error', 'An error occurred while loading PDF details.', true);
            }
        });
    });

    // Handle Pagination
    $(document).on('click', '.aipower-pdfs-page-btn', function() {
        const page = $(this).data('page');
        const searchTerm = $searchInput.val();

        UI.showSpinner();
        // Send AJAX request to refresh PDFs table
        $.ajax({
            url: ajaxurl,
            type: 'POST',
            data: {
                action: 'aipower_refresh_pdfs_table',
                page: page,
                search_term: searchTerm,
                _wpnonce: nonce
            },
            success: function(response) {
                UI.hideSpinner();
                if (response.success) {
                    $pdfsTableContainer.html(response.data.table);
                    UI.showMessage('success', response.data.message || 'PDFs table updated.', true);
                } else {
                    UI.showMessage('error', response.data.message, true);
                }
            },
            error: function() {
                UI.hideSpinner();
                UI.showMessage('error', 'An error occurred while refreshing the PDFs table.', true);
            }
        });
    });

    // Handle single delete confirmation prompt
    $(document).on('click', '.aipower-delete-pdf-icon', function(event) {
        event.stopPropagation(); // Prevent the click from propagating to the document

        const $currentRow = $(this).closest('tr');
        const $confirmation = $currentRow.find('.aipower-single-delete-pdf-confirmation');

        // Hide any other open confirmations
        $('.aipower-single-delete-pdf-confirmation').not($confirmation).hide();

        // Toggle the visibility of the confirmation prompt for this row
        $confirmation.toggle();

        if ($confirmation.is(':visible')) {
            // Attach a click handler to the document to hide the confirmation prompt when clicking outside
            $(document).on('click.aipower-single-delete-df', function(e) {
                if (!$(e.target).closest('.aipower-single-delete-pdf-confirmation').length && !$(e.target).is('.aipower-delete-pdf-icon')) {
                    $confirmation.hide();
                    // Remove this click handler to prevent multiple bindings
                    $(document).off('click.aipower-single-delete-pdf');
                }
            });
        } else {
            // If the confirmation prompt is hidden, remove the document click handler
            $(document).off('click.aipower-single-delete-pdf');
        }
    });

    // Prevent clicks inside the single delete confirmation prompt from propagating to the document
    $(document).on('click', '.aipower-single-delete-pdf-confirmation', function(event) {
        event.stopPropagation();
    });

    // Handle single delete "Yes" click
    $(document).on('click', '.aipower-single-confirm-pdf-yes', function() {
        const pdf_id = $(this).data('id');
        const $row = $(this).closest('tr');

        // Hide the confirmation prompt
        $row.find('.aipower-single-delete-pdf-confirmation').hide();
        // Remove the document click handler
        $(document).off('click.aipower-single-delete-pdf');

        // Proceed with deleting the PDF
        UI.showSpinner();

        // Send AJAX request to delete the PDF
        $.ajax({
            url: ajaxurl,
            type: 'POST',
            data: {
                action: 'aipower_delete_pdf',
                pdf_id: pdf_id,
                _wpnonce: nonce
            },
            success: function(response) {
                UI.hideSpinner();
                if (response.success) {
                    // Remove the PDF row from the table
                    $row.remove();
                    // Clear the PDF details if the deleted PDF was being viewed
                    $pdfDetailsContainer.empty();
                    UI.showMessage('success', response.data.message || 'PDF deleted successfully.', true);
                } else {
                    UI.showMessage('error', response.data.message, true);
                }
            },
            error: function() {
                UI.hideSpinner();
                UI.showMessage('error', 'An error occurred while deleting the PDF.', true);
            }
        });
    });
});