<?php
namespace WPAICG;

if ( ! defined( 'ABSPATH' ) ) exit;

if ( !class_exists( '\\WPAICG\\WPAICG_PDFs' ) ) {
    class WPAICG_PDFs {
        private static $instance = null;

        public static function get_instance() {
            if (is_null(self::$instance)) {
                self::$instance = new self();
            }
            return self::$instance;
        }

        public function __construct() {
            // AJAX actions
            add_action('wp_ajax_aipower_refresh_pdfs_table', array($this, 'aipower_refresh_pdfs_table'));
            add_action('wp_ajax_aipower_load_pdf_details', array($this, 'aipower_load_pdf_details'));
            add_action('wp_ajax_aipower_delete_pdf', array($this, 'aipower_delete_pdf'));
            add_action('wp_ajax_aipower_get_all_pdf_ids', array($this, 'aipower_get_all_pdf_ids'));
        }

        // This method returns ALL PDF IDs to the client
        public function aipower_get_all_pdf_ids() {
            if (!current_user_can('manage_options')) {
                wp_send_json_error(array('message' => 'You do not have sufficient permissions.'));
            }

            if (!isset($_POST['_wpnonce']) || !wp_verify_nonce($_POST['_wpnonce'], 'wpaicg_save_pdf_engine_nonce')) {
                wp_send_json_error(array('message' => 'Nonce verification failed.'));
            }

            global $wpdb;

            // Grab ALL PDFs of type wpaicg_pdfembed
            $pdf_ids = $wpdb->get_col(
                $wpdb->prepare("
                    SELECT ID 
                    FROM {$wpdb->posts} 
                    WHERE post_type = %s
                ", 
                'wpaicg_pdfembed')
            );

            if (empty($pdf_ids)) {
                wp_send_json_success(array(
                    'ids' => array(),
                    'message' => 'No PDFs found.'
                ));
            } else {
                wp_send_json_success(array(
                    'ids' => $pdf_ids
                ));
            }
        }

        public static function aipower_render_pdfs_table($paged = 1, $search_term = '') {
            // Ensure function only accessible to admins
            if (!current_user_can('manage_options')) {
                return '<p>' . esc_html__('You do not have sufficient permissions to access this page.', 'gpt3-ai-content-generator') . '</p>';
            }
        
            global $wpdb;
        
            // Number of PDFs to display per page
            $pdfs_per_page = 10;
        
            // Calculate the offset
            $offset = ($paged - 1) * $pdfs_per_page;
        
            // Prepare the base query
            $query = "SELECT ID, post_title, post_date FROM {$wpdb->posts} WHERE post_type = %s";
            $query_params = array('wpaicg_pdfembed');
        
            // Add search condition if search term is provided
            if (!empty($search_term)) {
                $query .= " AND (post_title LIKE %s OR post_content LIKE %s)";
                $like_term = '%' . $wpdb->esc_like($search_term) . '%';
                $query_params[] = $like_term;
                $query_params[] = $like_term;
            }
        
            // Get total number of PDFs
            $total_pdfs = $wpdb->get_var($wpdb->prepare(
                "SELECT COUNT(*) FROM {$wpdb->posts} WHERE post_type = %s" . (!empty($search_term) ? " AND (post_title LIKE %s OR post_content LIKE %s)" : ""),
                $query_params
            ));
        
            // Append ordering and limit to the query
            $query .= " ORDER BY post_date DESC LIMIT %d OFFSET %d";
            $query_params[] = $pdfs_per_page;
            $query_params[] = $offset;
        
            // Retrieve the PDFs
            $pdfs = $wpdb->get_results($wpdb->prepare($query, $query_params));
        
            // Calculate total pages
            $total_pages = ceil($total_pdfs / $pdfs_per_page);
        
            ob_start();
        
            if ($pdfs) {
                ?>
                <div id="aipower-pdfs-table-container">
                    <table class="aipower-pdfs-table">
                        <colgroup>
                            <col style="width: 8%;">
                            <col style="width: 25%;"> 
                            <col style="width: 45%;">
                            <col style="width: 12%;">
                        </colgroup>

                        <thead>
                            <tr>
                                <th><?php echo esc_html__('ID', 'gpt3-ai-content-generator'); ?></th>
                                <th><?php echo esc_html__('Title', 'gpt3-ai-content-generator'); ?></th>
                                <th><?php echo esc_html__('Details', 'gpt3-ai-content-generator'); ?></th>
                                <th><?php echo esc_html__('Actions', 'gpt3-ai-content-generator'); ?></th>
                            </tr>
                        </thead>
                        <tbody>
                            <?php
                            foreach ($pdfs as $pdf) {
                                // Retrieve meta fields
                                $token = get_post_meta($pdf->ID, 'wpaicg_embedding_token', true);
                                $ai = get_post_meta($pdf->ID, 'wpaicg_provider', true);
                                $model = get_post_meta($pdf->ID, 'wpaicg_model', true);
                                $index = get_post_meta($pdf->ID, 'wpaicg_index', true);

                                // Determine Vector DB based on index
                                if (strpos($index, 'pinecone.io') !== false) {
                                    $vector_db = 'Pinecone';
                                } elseif (empty($index)) {
                                    $vector_db = 'NA';
                                } else {
                                    $vector_db = 'Qdrant';
                                }

                                // Fallback to 'NA' if any meta field is empty
                                $token_display = !empty($token) ? esc_html($token) : esc_html__('NA', 'gpt3-ai-content-generator');
                                $ai_display = !empty($ai) ? esc_html($ai) : esc_html__('NA', 'gpt3-ai-content-generator');
                                $model_display = !empty($model) ? esc_html($model) : esc_html__('NA', 'gpt3-ai-content-generator');
                                $index_display = !empty($index) ? esc_html($index) : esc_html__('NA', 'gpt3-ai-content-generator');
                                $vector_db_display = !empty($vector_db) ? esc_html($vector_db) : esc_html__('NA', 'gpt3-ai-content-generator');

                                ?>
                                <tr>
                                    <td><?php echo esc_html($pdf->ID); ?></td>
                                    <td><?php echo self::truncate_text($pdf->post_title, 100); ?></td>
                                    <td>
                                        <div class="aipower-details">
                                            <strong><?php echo esc_html__('Token:', 'gpt3-ai-content-generator'); ?></strong> <?php echo $token_display; ?><br>
                                            <strong><?php echo esc_html__('AI:', 'gpt3-ai-content-generator'); ?></strong> <?php echo $ai_display; ?><br>
                                            <strong><?php echo esc_html__('Model:', 'gpt3-ai-content-generator'); ?></strong> 
                                            <?php echo self::truncate_text($model_display, 30); ?><br> <!-- Truncated Model -->
                                            <strong><?php echo esc_html__('Index:', 'gpt3-ai-content-generator'); ?></strong> 
                                            <?php echo self::truncate_text($index_display, 30); ?><br> <!-- Truncated Index -->
                                            <strong><?php echo esc_html__('Vector DB:', 'gpt3-ai-content-generator'); ?></strong> <?php echo $vector_db_display; ?><br>
                                            <strong><?php echo esc_html__('Date:', 'gpt3-ai-content-generator'); ?></strong> <?php echo esc_html(date_i18n(get_option('date_format') . ' ' . get_option('time_format'), strtotime($pdf->post_date))); ?>
                                        </div>
                                    </td>
                                    <td style="position: relative;">
                                        <span class="dashicons dashicons-visibility aipower-view-pdf-icon" data-id="<?php echo esc_attr($pdf->ID); ?>" title="<?php echo esc_attr__('View PDF', 'gpt3-ai-content-generator'); ?>"></span>
                                        <span class="dashicons dashicons-trash aipower-delete-pdf-icon" data-id="<?php echo esc_attr($pdf->ID); ?>" title="<?php echo esc_attr__('Delete PDF', 'gpt3-ai-content-generator'); ?>"></span>
                                        <!-- Inline confirmation prompt for single delete -->
                                        <div class="aipower-single-delete-pdf-confirmation" style="display: none;">
                                            <span><?php echo esc_html__('Sure?', 'gpt3-ai-content-generator'); ?></span>
                                            <span class="aipower-single-confirm-pdf-yes" data-id="<?php echo esc_attr($pdf->ID); ?>"><?php echo esc_html__('Yes', 'gpt3-ai-content-generator'); ?></span>
                                        </div>
                                    </td>
                                </tr>

                                <?php
                            }
                            ?>
                        </tbody>

                    </table>
        
                    <?php
                    // Show pagination only if there are multiple pages
                    if ($total_pages > 1) {
                        ?>
                        <div class="aipower-pdfs-pagination">
                            <?php
                            $range = 2; // Number of pages to show around the current page
                            $first_last = 2; // Always show the first and last two pages
        
                            if ($total_pages <= ($first_last * 2 + $range)) {
                                // If total pages are small, display all pages
                                for ($i = 1; $i <= $total_pages; $i++) {
                                    echo '<button class="aipower-pdfs-page-btn" data-page="' . esc_attr($i) . '" ' . ($i == $paged ? 'disabled' : '') . '>' . esc_html($i) . '</button>';
                                }
                            } else {
                                // Display the first set of pages
                                for ($i = 1; $i <= $first_last; $i++) {
                                    echo '<button class="aipower-pdfs-page-btn" data-page="' . esc_attr($i) . '" ' . ($i == $paged ? 'disabled' : '') . '>' . esc_html($i) . '</button>';
                                }
        
                                // Display ellipsis if needed
                                if ($paged > $first_last + $range + 1) {
                                    echo '<span>...</span>';
                                }
        
                                // Calculate start and end for middle pages
                                $start = max($first_last + 1, $paged - $range);
                                $end = min($paged + $range, $total_pages - $first_last);
        
                                // Display the middle pages
                                for ($i = $start; $i <= $end; $i++) {
                                    if ($i > $first_last && $i <= $total_pages - $first_last) {
                                        echo '<button class="aipower-pdfs-page-btn" data-page="' . esc_attr($i) . '" ' . ($i == $paged ? 'disabled' : '') . '>' . esc_html($i) . '</button>';
                                    }
                                }
        
                                // Display ellipsis if needed
                                if ($paged < $total_pages - $first_last - $range) {
                                    echo '<span>...</span>';
                                }
        
                                // Display the last set of pages
                                for ($i = $total_pages - $first_last + 1; $i <= $total_pages; $i++) {
                                    if ($i > $end) {
                                        echo '<button class="aipower-pdfs-page-btn" data-page="' . esc_attr($i) . '" ' . ($i == $paged ? 'disabled' : '') . '>' . esc_html($i) . '</button>';
                                    }
                                }
                            }
                            ?>
                        </div>
                        <?php
                    }
                    ?>
                </div>
                <?php
            } else {
                echo '<p>' . esc_html__('No PDFs found.', 'gpt3-ai-content-generator') . '</p>';
            }
        
            return ob_get_clean();
        }
        
        public function aipower_refresh_pdfs_table() {
            if (!current_user_can('manage_options')) {
                wp_send_json_error(array('message' => esc_html__('You do not have sufficient permissions to perform this action.', 'gpt3-ai-content-generator')));
                return;
            }
        
            if (!isset($_POST['_wpnonce']) || !wp_verify_nonce($_POST['_wpnonce'], 'wpaicg_save_pdf_engine_nonce')) {
                wp_send_json_error(array('message' => esc_html__('Nonce verification failed', 'gpt3-ai-content-generator')));
                return;
            }
        
            $paged = isset($_POST['page']) ? intval($_POST['page']) : 1; // Get current page
            $search_term = isset($_POST['search_term']) ? sanitize_text_field($_POST['search_term']) : '';
        
            // Use the pagination and search term when rendering the table
            $table_html = self::aipower_render_pdfs_table($paged, $search_term);
        
            wp_send_json_success(array('table' => $table_html));
        }
        

        public function aipower_load_pdf_details() {
            if (!current_user_can('manage_options')) {
                wp_send_json_error(array('message' => esc_html__('You do not have sufficient permissions to perform this action.', 'gpt3-ai-content-generator')));
                return;
            }

            if (!isset($_POST['_wpnonce']) || !wp_verify_nonce($_POST['_wpnonce'], 'wpaicg_save_pdf_engine_nonce')) {
                wp_send_json_error(array('message' => esc_html__('Nonce verification failed', 'gpt3-ai-content-generator')));
                return;
            }

            if (!isset($_POST['pdf_id']) || !is_numeric($_POST['pdf_id'])) {
                wp_send_json_error(array('message' => esc_html__('Invalid PDF ID.', 'gpt3-ai-content-generator')));
                return;
            }

            $pdf_id = intval($_POST['pdf_id']);
            $pdf_post = get_post($pdf_id);

            if (!$pdf_post || $pdf_post->post_type !== 'wpaicg_pdfembed') {
                wp_send_json_error(array('message' => esc_html__('PDF not found.', 'gpt3-ai-content-generator')));
                return;
            }

            // Build the PDF details HTML
            ob_start();
            ?>
            <div class="aipower-pdf-details">
                <h2><?php echo esc_html__('PDF Details', 'gpt3-ai-content-generator'); ?></h2>
                <p><?php echo esc_html($pdf_post->post_title); ?></p>
                <div class="aipower-pdf-content">
                    <?php
                    // Display the PDF content
                    echo wp_kses_post(wpautop($pdf_post->post_content));
                    ?>
                </div>
            </div>
            <?php
            $pdf_details = ob_get_clean();

            wp_send_json_success(array('pdf_details' => $pdf_details));
        }

        public function aipower_delete_pdf() {
            if (!current_user_can('manage_options')) {
                wp_send_json_error(array('message' => esc_html__('You do not have sufficient permissions to perform this action.', 'gpt3-ai-content-generator')));
                return;
            }
        
            if (!isset($_POST['_wpnonce']) || !wp_verify_nonce($_POST['_wpnonce'], 'wpaicg_save_pdf_engine_nonce')) {
                wp_send_json_error(array('message' => esc_html__('Nonce verification failed', 'gpt3-ai-content-generator')));
                return;
            }
        
            if (!isset($_POST['pdf_id']) || !is_numeric($_POST['pdf_id'])) {
                wp_send_json_error(array('message' => esc_html__('Invalid PDF ID.', 'gpt3-ai-content-generator')));
                return;
            }
        
            $pdf_id = intval($_POST['pdf_id']);
        
            \WPAICG\WPAICG_PDF::get_instance()->wpaicg_delete_embeddings_ids(array($pdf_id), 'wpaicg_pdfembed');
        
            wp_send_json_success(array('message' => esc_html__('PDF deleted successfully.', 'gpt3-ai-content-generator')));
        }

        /**
         * Truncate a string to a specified length and add ellipsis if needed.
         *
         * @param string $text The text to truncate.
         * @param int    $max_length The maximum length of the displayed text.
         * @return string The truncated text with a title attribute.
         */
        private static function truncate_text($text, $max_length = 100) {
            if (strlen($text) > $max_length) {
                $truncated = substr($text, 0, $max_length) . '...';
                return '<span class="aipower-truncated" title="' . esc_attr($text) . '">' . esc_html($truncated) . '</span>';
            } else {
                return esc_html($text);
            }
        }
    
    }
    WPAICG_PDFs::get_instance();
}
